#include "camion.h"
#include "screen.h"
#include "globdefs.h"
#include "random.h"
#include "spritesdef.h"
#include "sound.h"

extern TEXTURE tex_spr[];
extern PICTURE pic_spr[];
extern SPR_ATTR attr[];

extern FIXED ground_height[];

void camion_reinit(CAMION* c){
  c->level =  minmaxrndint(1,N_LEVEL-1);
  c->st = cs_still;
  c->posx = X_MIN-toFIXED(200.0);
}

void camion_init(CAMION* c){
  c->posx = X_MAX+X_MAX;
  c->invert=FALSE;
  camion_reinit(c);
}

void camion_display(CAMION* c){
  FIXED pos[XYZSS];
  static ANGLE bang=DEGtoANG(0.0);

  pos[X] = c->posx;
  pos[Y] = ground_height[c->level];//+toFIXED(+16.0-4.0);
  pos[Z] = toFIXED(168.0);
  pos[S] = (c->invert)?toFIXED(-1.0):toFIXED(+1.0);
  pos[4] = toFIXED(1.0);

  attr[SP_CAMION1].atrb =  CL256Bnk;
  attr[SP_CAMION2].atrb =  CL256Bnk;
  attr[SP_CAMION3].atrb =  CL256Bnk;

  slDispSpriteHV(pos,(SPR_ATTR *)(&attr[SP_CAMION1]),bang);   
  
  pos[X] = c->posx+((c->invert)?toFIXED(-32.0):toFIXED(32.0));
  slDispSpriteHV(pos,(SPR_ATTR *)(&attr[SP_CAMION2]),bang);  
  pos[X] = c->posx+((c->invert)?toFIXED(-64.0):toFIXED(64.0));
  slDispSpriteHV(pos,(SPR_ATTR *)(&attr[SP_CAMION3]),bang); 
  pos[X] = c->posx+((c->invert)?toFIXED(-96.0):toFIXED(96.0));
  slDispSpriteHV(pos,(SPR_ATTR *)(&attr[SP_CAMION2]),bang);  
  pos[X] = c->posx+((c->invert)?toFIXED(-128.0):toFIXED(128.0));
  slDispSpriteHV(pos,(SPR_ATTR *)(&attr[SP_CAMION3]),bang); 
}

void camion_arm(CAMION* c){
  if (c->st==cs_still){
    c->st = cs_armed;
  }
}

// returns drop
int camion_run(CAMION* c){

  int drop = -1;
  switch (c->st){
  case cs_still :
    break;
  case cs_running:
    c->posx += c->invert?toFIXED(2.0):-toFIXED(2.0);
    if (((c->posx)>(X_MAX+toFIXED(220.0)))||((c->posx)<(X_MIN-toFIXED(220.0)))){
      c->invert = !(c->invert);
      c->st = cs_still;
      c->level = minmaxrndint(0,N_LEVEL-1);
    } else {
      if ((c->posx>toFIXED(-150.0))&&(c->posx<toFIXED(150.0))){
	
	if (slRandom()>toFIXED(0.99))
	  drop = 1;
      }
    }
    break;
  case cs_armed :
    sound_tell(vs_sirene,1);
    c->st = cs_running;
    break;
  }
    return drop;
}

int camion_hits(CAMION*c, FIXED posx, FIXED level,FIXED* nposx){
  FIXED l,r;

  if (c->level != level)
    return FALSE;
  
  if (c->invert){
    r =  c->posx+toFIXED(16.0);
    l = r-toFIXED(5.0*32.0);
  } else {
    l = c->posx-toFIXED(16.0);
    r = l+toFIXED(5.0*32.0);  
  }

  if ((posx<l)||(posx>r))
    return FALSE;
  
  *nposx = (c->invert)?r:l;  
  return TRUE;
}
