#include "gameobj.h"
#include "globdefs.h"
#include "spritesdef.h"
#include "screen.h"
#include "random.h"
#include "sound.h"
#include "stats.h"
#include "expl.h"


#define TIME_MUL 1
#define TIME_ADD 0
#define DEFAULT_TIME_LIMIT 4*60*50
//#define DEFAULT_TIME_LIMIT 1000 

extern TEXTURE tex_spr[];
extern PICTURE pic_spr[];
extern SPR_ATTR attr[];

extern FIXED spr_offset[][2];

FIXED ground_height[]={
  -toFIXED(32.0) + LEVEL_FIRST,
  -toFIXED(32.0) + LEVEL_FIRST-1*LEVEL_HEIGHT,
  -toFIXED(32.0) + LEVEL_FIRST-2*LEVEL_HEIGHT,
  -toFIXED(32.0) + LEVEL_FIRST-3*LEVEL_HEIGHT,
  -toFIXED(32.0) + LEVEL_FIRST-4*LEVEL_HEIGHT,
};

#define MAX_ENERGY 40

// gives the sprite attribute index for each player state

int state_sprite[] = {
  0,             // static
  1,2,3,4,5,     // run
  6,7,8,          // down
  7,6,           // up
  9,10,11,12,13, // hikick
  14,15,16,      // walk
  17,18,19,18,   // downhit
  6,7,8,7,6,     // jump
  32,33,34,35,36,// level up
  37,38,37,32,   // hang
  39,40,41,42,43,// hi hand
  44,45,46,7,    // h f
  47,48,49,50,51,// n s h
  47,48,         // sa h l
  44,44,45,46,7, // hit hh run 
  72,73,74,75,76,77, // gard arr
  78,79,80,81,   // pfixe 
  82,83,84,85,84,83,   // hangs
};


// timings
#define T_1 3
#define T_2 4
#define T_3 6
#define T_300 (50*8)

/* the collision part */
 
// distancies 
#define D_NO 0x0 //contact
#define D_1  0x1
#define D_2  0x2
#define D_3  0x3 // far

FIXED distancy[]={
  toFIXED(0.0),
  toFIXED(6.0),
  toFIXED(16.0),
  toFIXED(25.0),
};

// direction
#define Z_L (0x1<<4) // left 

// heights
#define P_HH 0x8
#define P_H  0x4
#define P_M  0x2
#define P_L  0x1

/* hit and safe structures 

HIT :

x Uint 8  8  7  6  5   4  3  2  1 
x                     [__________] attack heights (P_HH|P_H|P_M|P_L)
x                 [_]              attack direction  Z_L : left
x        [_______]                 attack distancy 


SAFE & hurt :

x Uint 8  8  7  6  5   4  3  2  1 
x                     [__________] protection heights for right (P_HH|P_H|P_M|P_L)
x        [__________]              protection heights for left  (P_HH|P_H|P_M|P_L)



*/

Uint8 safe[ps_player_state_last];
Uint8 hit[ps_player_state_last];
Uint8 ener[ps_player_state_last];  // attack energy (if some)

inline Uint8 makehit(Uint8 dist, Uint8 dir, Uint8 hflags){
  return (((0x7&dist)<<5)|(Z_L&dir)|(0xF&hflags));
}

inline Uint8 revhit(Uint8 s){
  //  return (s&(~(s&(Z_L<<4))));
  return (s^Z_L);
}
inline Uint8 makesafe(Uint8 lflags, Uint8 rflags){
  return (((0xF&lflags)<<4)|(0xF&rflags));
}

inline Uint8 revsafe(Uint8 s){
  return (((0xF&s)<<4)|((0xF0&s)>>4));
}

int hit_can_hit_dist(FIXED p1x, FIXED p2x,Uint8 p1hit){
  Uint8 disti = (Uint8) (0x7&(p1hit>>5)); 
  int dir =  (p1hit & Z_L);
  Uint8 can = FALSE;

  if (dir) { //left
    if (p2x<p1x){
      can = ((p1x-p2x)<distancy[disti]);
    }
  } else {
    if (p1x<p2x){
      can = ((p2x-p1x)<distancy[disti]);
    }
  }
  return can;
}

int hit_can_hit_protection(Uint8 lhit, Uint8 lsafe){
  if (lhit & Z_L){
    lhit = lhit&0x0F;
  } else {
    lhit = (lhit<<4)&0xF0;
  }

  return (lhit & (~lsafe));
}

void player_get_high(PLAYER* p, FIXED* y){
  *y = ground_height[p->level];
}

inline int player_is_ko(PLAYER* p){
  return (p->state==ps_nsh5);
}

int player_can_take(PLAYER* p, FIXED posx, int level){

  if (p->state != ps_down3)
    return FALSE;
  if (p->level != level)   
    return FALSE;
   
  FIXED dist = p->posx - posx;      

  if (dist > toFIXED( 8.0)) 
    return FALSE;
  if (dist < toFIXED(-8.0)) 
    return FALSE;

  return TRUE;
}

void player_collide(PLAYER* p1, PLAYER* p2){
 
  Uint8 lhit,lsafe;
  Uint8 lhurt;

  if (p1->level != p2->level)
    return;

  lhit = (p1->left)?hit[p1->state]:revhit(hit[p1->state]);

  // distancy 
  if (!hit_can_hit_dist(p1->posx,p2->posx,lhit))
    return;

  lsafe    =  (p2->left)?safe[p2->state]:revsafe(safe[p2->state]);

  //protection
  lhurt = hit_can_hit_protection(lhit,lsafe);
  if (!lhurt)
    return ;
  
  p2->damage.zone     |= lhurt;
  p2->damage.energy   += ener[p1->state];
  p2->damage.p  = (void*) p1;
  p1->score +=  ener[p1->state];
}

void init_collision_data(){

  int i;
  for (i=0;i<ps_player_state_last;i++){
    safe[i]=makesafe(P_HH,P_HH);
    hit[i]=0x0;
    ener[i]=0x0;
  }
  
  hit[ps_hikick2] = makehit(D_2,Z_L,P_M);  ener[ps_hikick2] = 2;
  hit[ps_hikick3] = makehit(D_3,Z_L,P_H);  ener[ps_hikick3] = 3;
  hit[ps_downhit2] = makehit(D_1,Z_L,P_L);  ener[ps_downhit2] = 1;
  hit[ps_downhit3] = makehit(D_3,Z_L,P_L);  ener[ps_downhit3] = 2;
  //  hit[ps_levelup3] = makehit(D_1,Z_L,P_L);  ener[ps_levelup3] = 1;
  hit[ps_hihand3] = makehit(D_2,Z_L,P_H);  ener[ps_hihand3] = 2;
  hit[ps_hihand4] = makehit(D_3,Z_L,P_H);  ener[ps_hihand4]= 2;
  hit[ps_hf1] = makehit(D_1,Z_L,P_H);  ener[ps_hf1] = 3;
  hit[ps_hf2] = makehit(D_3,Z_L,P_H|P_HH);  ener[ps_hf2] = 3;

  hit[ps_hihhr1] = makehit(D_1,Z_L,P_M); ener[ps_hihhr2] = 2;
  hit[ps_hihhr2] = makehit(D_1,Z_L,P_M); ener[ps_hihhr2] = 3;
  hit[ps_hihhr3] = makehit(D_3,Z_L,P_H|P_M); ener[ps_hihhr3] = 5;


  hit[ps_pfixe2] = makehit(D_1,Z_L,P_H);  ener[ps_pfixe2] = 1;
  hit[ps_pfixe3] = makehit(D_3,Z_L,P_H);  ener[ps_pfixe3]= 3;

  hit[ps_hangs1] = makehit(D_2,Z_L,P_HH); ener[ps_pfixe3]= 2;

  safe[ps_quiet] = makesafe(P_HH,P_HH);

  safe[ps_down2] = makesafe(P_HH|P_H,P_HH|P_H);  
  safe[ps_down3] = makesafe(P_HH|P_H,P_HH|P_H);

  safe[ps_up1] = makesafe(P_HH|P_H,P_HH|P_H);  
  
  
  safe[ps_downhit1] =
    safe[ps_downhit2] = 
    safe[ps_downhit3] = 
    safe[ps_downhit4] =  makesafe(P_HH|P_H,P_HH|P_H);  



  safe[ps_jump2] = makesafe(P_L|P_HH,P_L|P_HH);  
  safe[ps_jump3] = makesafe(P_L|P_HH,P_L|P_HH);  
  safe[ps_jump4] = makesafe(P_L|P_HH,P_L|P_HH);  
 
  safe[ps_levelup1] = makesafe(P_HH,P_HH);  
  safe[ps_levelup2] = makesafe(P_HH|P_H|P_M|P_L,P_HH|P_H|P_M|P_L);  
  safe[ps_levelup3] = makesafe(P_HH|P_H|P_M|P_L,P_HH|P_H|P_M|P_L);  
  safe[ps_levelup4] = makesafe(P_HH|P_H|P_M|P_L,P_HH|P_H|P_M|P_L);  
  safe[ps_levelup5] = makesafe(P_HH|P_H|P_M,P_HH|P_H);

  
  safe[ps_hang1] = makesafe(P_H|P_M|P_L,P_H|P_M|P_L);  
  safe[ps_hang2] = makesafe(P_H|P_M|P_L,P_H|P_M|P_L);  
  safe[ps_hang3] = makesafe(P_H|P_M|P_L,P_H|P_M|P_L);
  safe[ps_hang4] = makesafe(P_H|P_M|P_L,P_H|P_M|P_L);

  safe[ps_hf1] = makesafe(P_L|P_M,P_L|P_M);  
  safe[ps_hf2] = makesafe(P_L|P_M,P_L|P_M);  
  safe[ps_hf3] = makesafe(P_L|P_M,P_L|P_M);  
  safe[ps_hf4] = makesafe(P_HH,P_HH);  

  safe[ps_nsh1] = makesafe(P_HH|P_H,P_HH);  
  safe[ps_nsh2] = makesafe(P_HH|P_H,P_HH);  
  safe[ps_nsh3] = makesafe(P_HH|P_H|P_M,P_HH);  
  safe[ps_nsh4] = makesafe(P_HH|P_H|P_M|P_L,P_HH|P_H|P_M|P_L);  
  safe[ps_nsh5] = makesafe(P_HH|P_H|P_M|P_L,P_HH|P_H|P_M|P_L);  
  
  safe[ps_sahl1] = makesafe(P_HH|P_H,P_HH); 
  safe[ps_sahl2] = makesafe(P_HH|P_H,0);  

  safe[ps_hihhr1] = safe[ps_hihhr4] = safe[ps_hihhr5] = makesafe(P_HH|P_L,P_HH|P_L);
  safe[ps_hihhr2]=  safe[ps_hihhr4] = makesafe(P_L|P_M|P_H,P_L|P_M|P_H);

  safe[ps_gardarr2] =safe[ps_gardarr3] = safe[ps_gardarr4] 
    = safe[ps_gardarr5] = safe[ps_gardarr6] =  makesafe(P_HH|P_H|P_M,P_HH);

  safe[ps_hangs1] = makesafe(P_H|P_M|P_L,P_H|P_M|P_L);  
  safe[ps_hangs2] = makesafe(P_H|P_M|P_L,P_H|P_M|P_L);  
  safe[ps_hangs3] = makesafe(P_H|P_M|P_L,P_H|P_M|P_L);
  safe[ps_hangs4] = makesafe(P_H|P_M|P_L,P_H|P_M|P_L);
  safe[ps_hangs5] = makesafe(P_H|P_M|P_L,P_H|P_M|P_L);
  safe[ps_hangs6] = makesafe(P_H|P_M|P_L,P_H|P_M|P_L);
}


/* MARK */
	
void mark_init(MARK * m,int spr_num){
  m->pos[X] = toFIXED(0.0);
  m->pos[Y] = toFIXED(0.0);
  m->spr_num = spr_num;
}


void mark_reinit(MARK* m){
  m->pos[X] = toFIXED(0.0);
  m->pos[Y] = toFIXED(0.0);
}
  
/* DAMAGE */

void damage_copy(DAMAGE* s, DAMAGE* d){
  d->energy = s->energy;
  d->zone = s->zone;
  d->p = s->p;
}

void damage_reset(DAMAGE* d){
  d->energy = 0;
  d->zone = 0x0;
  d->p = (void*)0;
}

/* PLAYER */



void player_reinit(PLAYER* p){
  p->posy=toFIXED(0.0);
  p->posx=minmaxrnd(X_MIN+toFIXED(30.0),X_MAX-toFIXED(30.0));
  p->level=minmaxrndint(0,N_LEVEL-1);
  p->state = ps_quiet;
  p->energy = MAX_ENERGY;
  p->mojo = 100;
  p->st_timeout = 1;
  p->damage.energy = 0;
  p->damage.zone= (Uint8) 0x0;
  p->damage.p = (void*)0;
  damage_copy(&(p->damage),&(p->last_damage));
  mark_reinit(&(p->m));
  p->score = 0;
  p->ndied = 0;
  p->size = toFIXED(1.0);

  p->cipiu.to = -1;
  p->cipiu.padd = 0;
  p->cipiu.padp = 0;

}

void player_init(PLAYER* p){
  p->active=FALSE;
  p->pad_num=-1;
  p->team=-1;
  p->left=slRandom()&0x1;

  player_reinit(p);
}

void player_change_team(PLAYER* p,int team){
  p->team = team%MAX_TEAM;
  p->m.spr_num = FIRST_TEAM_MARK+p->team;
}

inline void player_return(PLAYER* p){
  p->left = !(p->left);
}

inline void player_set_state_and_pos(PLAYER*p,player_state st,int to,FIXED dx,FIXED dy){
  p->state = st;
  p->st_timeout = (to+TIME_ADD)*TIME_MUL;
  p->posx += (p->left)?dx:-dx;
  p->posy += dy;
}

void player_check_boundaries(PLAYER* p){
  if (p->posx>(X_MAX)){
    //    player_set_state_and_pos(p,ps_quiet,T_3,0,0);
    //    p->posx = X_MIN;
    p->posx = X_MAX;
  }  else if (p->posx<X_MIN){
    //    player_set_state_and_pos(p,ps_quiet,T_3,0,0);
    //    p->posx = X_MAX;
    p->posx = X_MIN;
  }
}



void player_apply_damage(PLAYER* p,int gopt){
  Uint8 zone = p->damage.zone;

  if ((p->damage.energy != 0)
      &&(p->damage.zone != p->last_damage.zone)){
    
    if (gopt & GA_TEAM){
      PLAYER* offp = ((PLAYER*) (p->damage.p));
      
      if ((offp->team)!=(p->team)){
	switch (p->team) {
	case 0 : // living
	  if (offp->team == 1) p->energy -= p->damage.energy;
	  if (offp->team == 2) p->energy -= p->damage.energy;
	  break;
	case 1 : // dead
	  if (offp->team == 0) p->energy += p->damage.energy;
	  if (offp->team == 2) p->energy += p->damage.energy;
	  break;
	case 2 : // living dead
	default:	  
	  if (offp->team == 0) p->energy += p->damage.energy;
	  if (offp->team == 1) p->energy -= p->damage.energy;
	  break;
	}
      }
    } else {
      p->energy -= p->damage.energy;
    }
    
    stats_add(1);
    
    FIXED expl_y;
    FIXED expl_x;

    /*   if (zone>0xF){
	 expl_x = p->posx-toFIXED(5.0);
	 } else {
	 expl_x=  p->posx+toFIXED(5.0);
	 }
    */
    expl_x = p->posx+minmaxrndint(-toFIXED(3.0),toFIXED(3.0));

    if (zone&0x88){
      expl_y = ground_height[p->level]-toFIXED(20.0)+minmaxrnd(-toFIXED(3.0),toFIXED(3.0));
      expl_add(expl_x,expl_y);
    }
    if (zone&0x44){
      expl_y = ground_height[p->level]-toFIXED(10.0)+minmaxrnd(-toFIXED(3.0),toFIXED(3.0));
      expl_add(expl_x,expl_y);
    }
    if (zone&0x22){
      expl_y = ground_height[p->level]+minmaxrnd(-toFIXED(3.0),toFIXED(3.0));
      expl_add(expl_x,expl_y);
    }
    if (zone&0x11){
      expl_y = ground_height[p->level]+toFIXED(10.0)+minmaxrnd(-toFIXED(3.0),toFIXED(3.0));
      expl_add(expl_x,expl_y);
    }

    switch (p->state){
    default :
      if (zone & P_HH){
      } else {
	sound_tell(minmaxrndint(vs_hit1,vs_hit3),2);
	if (zone & (P_HH|P_H|P_M|P_L)){
	  p->posx -=  slMulFX(toFIXED(0.3*p->damage.energy),distancy[D_1]);
	  p->m.pos[X] -=  slMulFX(toFIXED(1.0*p->damage.energy),distancy[D_1]);
	} else {
	  p->posx += slMulFX(toFIXED(0.3*p->damage.energy),distancy[D_1]);
	  p->m.pos[X] += slMulFX(toFIXED(1.0*p->damage.energy),distancy[D_1]);
	}
	if (gopt & GA_TEAM){
	  if ((p->energy)<0){
	    player_set_state_and_pos(p,ps_nsh1,T_1,0,0);
	    sound_tell(vs_die,1);
	    if ((p->team) == 0) p->team = 2; // living -> living dead
	    if ((p->team) == 2) p->team = 1; // living-dead  -> dead				  
	    player_change_team(p,p->team);
	  }
	  else if ((p->energy)>(2*MAX_ENERGY)){
	    player_set_state_and_pos(p,ps_nsh1,T_1,0,0);
	    sound_tell(vs_die,1);
	    if ((p->team) == 1) p->team = 2; // dead -> living dead
	    if ((p->team) == 2) p->team = 0; // living-dead  -> living				  
	    player_change_team(p,p->team);
	  }
	}else {	
	  if (p->energy<0){
	    player_set_state_and_pos(p,ps_nsh1,T_1,0,0);
	    p->ndied++;
	    sound_tell(vs_die,1);
	  }
	}
      }
    }
  }

  damage_copy(&(p->damage),&(p->last_damage));
  damage_reset(&(p->damage));
}

void player_change_level(PLAYER* p,int s){
  int tl = p->level+s;
  p->level = (tl<0)?N_LEVEL-1:(tl>=N_LEVEL)?0:tl;
}

void player_fall(PLAYER* p){
  if (p->posy<toFIXED(0.0)){
    p->posy -= slMulFX(toFIXED(0.58),p->posy);
    if (p->posy>toFIXED(-1.0))
      p->posy = toFIXED(0.0);
  } else {
    p->posy = toFIXED(0.0);
  }
}

int mark_period = 0;

void player_mark_display(PLAYER* p){
  FIXED pos[XYZSS];
  static ANGLE bang=DEGtoANG(0.0);
  FIXED lsin;
  int spr_num = p->m.spr_num  ;
 
  //  bang+=DEGtoANG(2.0);

  p->m.pos[X] += slMulFX(toFIXED(0.05),(p->posx - p->m.pos[X]));
  p->m.pos[Y] += slMulFX(toFIXED(0.05),(ground_height[p->level]+toFIXED(30.0)- p->m.pos[Y]));
  
  //  toFIXED(p->energy/(MAX_ENERGY*1.0)
  mark_period+=2;
  if ((mark_period)>360) mark_period = 0;
  lsin = slSin(mark_period*(p->energy/(MAX_ENERGY*1.0))*90.0);
  
  pos[X] =  p->m.pos[X];
  pos[Y] =  p->m.pos[Y];
  pos[Z] = toFIXED(175.0);
  //  pos[S] = toFIXED(p->energy/(MAX_ENERGY*1.0));
  //  pos[4] = toFIXED(p->energy/(MAX_ENERGY*1.0));

  
  pos[S] = toFIXED(0.1)+slMulFX(lsin,toFIXED(1.0));
  pos[4] = toFIXED(0.1)+slMulFX(lsin,toFIXED(1.0));

  attr[spr_num].atrb = CL256Bnk|CL_Gouraud;//|MESHon;
  attr[spr_num].gstb = p->gr_col;

  slDispSpriteHV(pos,(SPR_ATTR *)(&attr[spr_num]),bang);   
}

void player_display(PLAYER* p){
  FIXED pos[XYZSS];
  ANGLE ang;
 
  int spr_num = state_sprite[p->state];
  if (p->left){
    pos[S] = slMulFX(toFIXED(1.0),p->size);
    pos[X]= p->posx-spr_offset[spr_num][X];
  } else {
    pos[S] = slMulFX(toFIXED(-1.0),p->size);
    pos[X]= p->posx+spr_offset[spr_num][X];
  }
  pos[Y]= p->posy-spr_offset[spr_num][Y]+ground_height[p->level];
  pos[Z] = toFIXED(169.0);
  pos[4] = p->size;
  
  ang = DEGtoANG(0.0);
  attr[spr_num].gstb = p->gr_col;
  if (p->team == 2){
    attr[spr_num].atrb = CL256Bnk|CL_Gouraud|MESHon;
  } else {
    attr[spr_num].atrb = CL256Bnk|CL_Gouraud;
  }
  slDispSpriteHV(pos,(SPR_ATTR *)(&attr[spr_num]),ang);   

  player_mark_display(p);


#ifdef UNDEF

  // display safe 
  pos[Z] = toFIXED(200.0);
  pos[S] = toFIXED(0.5);
  pos[S] = toFIXED(1.0);  
  pos[X] =  p->posx+toFIXED(10.0);
  pos[Y] =  ground_height[p->level]-toFIXED(20.0);
  attr[p->m.spr_num].gstb = p->gr_col;

  ANGLE bang=0x0;

  // display safe;
  Uint8 lsafe;
  if (p->left){
    lsafe = safe[p->state];
  } else {
    lsafe = revsafe(safe[p->state]);
  }

  if (lsafe & (P_HH)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }
  pos[Y] +=toFIXED(11.0);
  if (lsafe & (P_H)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }
  pos[Y] +=toFIXED(11.0);
  if (lsafe & (P_M)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }
  pos[Y] +=  toFIXED(11.0);
  if (lsafe & (P_L)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }

  pos[X] =  p->posx-toFIXED(10.0);
  pos[Y] =  ground_height[p->level]-toFIXED(20.0);

  if (lsafe & (P_HH<<4)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }
  pos[Y] +=toFIXED(11.0);
  if (lsafe & (P_H<<4)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }
  pos[Y] +=toFIXED(11.0);
  if (lsafe & (P_M<<4)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }
  pos[Y] += toFIXED(11.0);
  if (lsafe & (P_L<<4)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }
  Uint8 lhit;
  
  // display hit

  if (p->left){
    lhit = hit[p->state];
  } else {
    lhit = revhit(hit[p->state]);
  }

  if (lhit & (1<<4)) 
    pos[X] =   p->posx-toFIXED(20.0);
  else 
    pos[X] =   p->posx+toFIXED(20.0);
  pos[Y] =  ground_height[p->level]-toFIXED(20.0);

  if (lhit & (P_HH)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }
  pos[Y] += toFIXED(11.0);
  if (lhit & (P_H)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }
  pos[Y] += toFIXED(11.0);
  if (lhit & (P_M)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   
  }
  pos[Y] += toFIXED(11.0);
  if (lhit & (P_L)){
    slDispSprite(pos,(SPR_ATTR *)(&attr[p->m.spr_num].texno),bang);   

  }
#endif 
  /*
    pos[X] = p->posx-toFIXED(40);
    for (i=0;i<8;i++){
    pos[Y] =  ground_height[p->level]+toFIXED(50.0);
    if (safe[p->state]&(1<<i)) slDispSprite(pos,(SPR_ATTR *)(&attr[FIRST_MARK].texno),bang);  
    pos[Y] += toFIXED(10.0);
    if (hit[p->state]&(1<<i)) slDispSprite(pos,(SPR_ATTR *)(&attr[FIRST_MARK].texno),bang);  
    pos[Y] += toFIXED(10.0);
    if (ener[p->state]&(1<<i)) slDispSprite(pos,(SPR_ATTR *)(&attr[FIRST_MARK].texno),bang);  

    pos[X]+=toFIXED(10);
    
    }
  */
}



#define K_FOOT    PER_DGT_TA
#define K_HAND    PER_DGT_TX
#define K_PROTECT PER_DGT_TC
#define K_RUN     PER_DGT_TB

void player_cipiu(PLAYER* p, char ga, GAME* g){

  CIPIU* cipiu = &(p->cipiu);
  PLAYER *target;

  if ((cipiu->to)<0){ // think
    
    game_set_closer_player(g,p,&(target));
    p->cipiu.target = (void*) target;

    if (target){

      int same_team ;
      if (ga & GA_TEAM)
	same_team = (target->team) == (p->team);
      else 
	same_team = 0;

      if (same_team){
	cipiu->goal = (slRandom()>toFIXED(0.5))?cg_follow:cg_rest;
	cipiu->to=minmaxrndint(20,400);
      } else {
	if (slRandom()>toFIXED(0.3)){
	  cipiu->goal = cg_follow;
	  cipiu->to=minmaxrndint(20,300);
	} else 	if (slRandom()>toFIXED(0.5)){
	  cipiu->goal= cg_runaway;
	  cipiu->to=minmaxrndint(20,120);
	} else {
	  cipiu->goal= cg_defend;
	  cipiu->to=minmaxrndint(50,100);
	}
      }

    } else {
      cipiu->goal =  (slRandom()>toFIXED(0.7))?cg_rest:(slRandom()>toFIXED(0.5))?cg_patrol1:cg_patrol2;
      cipiu->to=minmaxrndint(10,100);
    }        
    return;
  }

  (p->cipiu.to)--;

  // act 
  
  FIXED dist=0;
  int friend=0;
  target = (PLAYER*)(cipiu->target);

  if (target) {
    dist = target->posx - p->posx;
      if (ga & GA_TEAM)
	friend = (target->team) == (p->team);
      else 
	friend = 0;
  }
  
  switch (cipiu->goal){
    
  case cg_patrol1 :
      p->cipiu.padd = PER_DGT_KL;
      if (p->posx<X_RMIN) cipiu->goal = cg_patrol2;
    break ;

  case cg_patrol2 :
      p->cipiu.padd = PER_DGT_KR;
      if (p->posx>X_RMAX) cipiu->goal = cg_patrol1;
    break ;

  case cg_defend : // tourner
    p->cipiu.padd =  K_FOOT|PER_DGT_KD;
    break;

  case cg_follow :
    
    if (dist>0) {
      p->cipiu.padd =  PER_DGT_KR;
      if (!friend){
	if (dist<distancy[D_1]){
	  p->cipiu.padd |= K_HAND;
	} else if (dist<distancy[D_2]){
	  p->cipiu.padd |= K_FOOT|PER_DGT_KD;
	} else if (dist<distancy[D_3]){
	  p->cipiu.padd |= K_FOOT;
	  //	  if (dist>toFIXED(30.0)) p->cipiu.padd |= K_RUN;
	}
      }
    } else {
      p->cipiu.padd =  PER_DGT_KL;
      
      if (!friend){
	if (dist>-distancy[D_1]){
	  p->cipiu.padd |= K_HAND;
	} else if (dist>-distancy[D_2]){
	  p->cipiu.padd |= K_FOOT|PER_DGT_KD;
	} else if (dist>-distancy[D_3]){
	  p->cipiu.padd |= K_FOOT;
	}
      }
      //	  if (dist>-toFIXED(30.0)) p->cipiu.padd |= K_RUN;
    }
    break;
  case cg_runaway :
    p->cipiu.padd = K_RUN;

    if ((p->posx>X_RMAX)||(p->posx<X_RMIN)) cipiu->to =-1;

    if (dist>0) {
      p->cipiu.padd |=  PER_DGT_KL;
    } else {
      p->cipiu.padd |=  PER_DGT_KR;
    }
    break;
  case cg_rest :
  default :
    p->cipiu.padd =  PER_DGT_KD;
    break ;
  }
}




void player_update_from_control(PLAYER* p,char ga, GAME* g){
  Uint16 padp,padd;

  if (p->pad_num != -1){
    padp = ~(Smpc_Peripheral[p->pad_num].push);
    padd = ~(Smpc_Peripheral[p->pad_num].data);    
  } else { //machine
    player_cipiu(p,ga,g);
    padp = p->cipiu.padp;
    padd = p->cipiu.padd;
  } 

  if (!(p->left)){
    if (padd & PER_DGT_KL){
      padd &= ~PER_DGT_KL;
      padd |= PER_DGT_KR;
    } else if (padd & PER_DGT_KR){
      padd &= ~PER_DGT_KR;
      padd |= PER_DGT_KL;
    }
  }

  switch (p->state){
  case ps_quiet:
    player_fall(p);
    if ((p->st_timeout) == 0){
      player_set_state_and_pos(p,ps_quiet,T_2,0,0);
      if (padd & K_PROTECT) {
	if (padd & PER_DGT_KU){
	  player_set_state_and_pos(p,ps_sahl1,T_2,0,0);
	} else 	if (padd & PER_DGT_KR){
	  player_set_state_and_pos(p,ps_gardarr1,T_2,0,0);
	}
      }
      else if (padd & K_HAND) {
	if (padd & PER_DGT_KL) {
	  player_set_state_and_pos(p,ps_hihand1,T_2,toFIXED(-5.0),0);
	} else {
	  player_set_state_and_pos(p,ps_pfixe1,T_2,toFIXED(-2.0),0);
	}

	      
      }
      else if ((padd & PER_DGT_KL)&&(padd & K_RUN)) player_set_state_and_pos(p,ps_run1,T_1,0,0);
      else if (padd & PER_DGT_KU) player_set_state_and_pos(p,ps_jump1,T_2,0,0);
      else if (padd & K_FOOT) player_set_state_and_pos(p,ps_hikick1,T_2,0,0);
      else if (padd & PER_DGT_KL) player_set_state_and_pos(p,ps_walk1,T_2,toFIXED(-3.0),0);
      else if (padd & PER_DGT_KR) { 
	player_return(p);
	player_set_state_and_pos(p,ps_quiet,T_2,0,0);
      }

      else if (padd & PER_DGT_KD) player_set_state_and_pos(p,ps_down1,T_2,0,0);
    
    }break;

  case ps_run1:
    if ((p->st_timeout) == 0){ 
      player_set_state_and_pos(p,ps_quiet,T_2,0,0);
      if (padd & PER_DGT_KR) player_set_state_and_pos(p,ps_walk1,T_2,0,0);
      else if (padd & PER_DGT_KL) player_set_state_and_pos(p,ps_run2,T_1,toFIXED(-5.0),0);
    }break;
  case ps_run2:
    if ((p->st_timeout) == 0){ 
      if (padd & PER_DGT_KR) player_set_state_and_pos(p,ps_walk1,T_2,0,0);
      else player_set_state_and_pos(p,ps_run3,T_1,toFIXED(-5.0),0);
    }break;
  case ps_run3:
    if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR) player_set_state_and_pos(p,ps_walk1,T_2,0,0);
      else player_set_state_and_pos(p,ps_run4,T_1,toFIXED(-6.0),0);
      sound_tell(minmaxrndint(vs_step1,vs_step6),10);
    }break;
  case ps_run4:
    if ((p->st_timeout) == 0){ 
      player_set_state_and_pos(p,ps_quiet,T_2,0,0);
      if ((padd & K_FOOT)&&(padd & K_RUN))player_set_state_and_pos(p,ps_hihhr1,T_2,toFIXED(-2.0),0);
      else if (padd & PER_DGT_KR) player_set_state_and_pos(p,ps_walk1,T_2,toFIXED(-2.0),0);
      else if (padd & PER_DGT_KL) player_set_state_and_pos(p,ps_run5,T_1,toFIXED(-5.0),0);
    }break;
  case ps_run5:
    if ((p->st_timeout) == 0){ 
      player_set_state_and_pos(p,ps_quiet,T_2,0,0);
      if (padd & PER_DGT_KL) player_set_state_and_pos(p,ps_run1,T_1,toFIXED(-2.0),0);
    }break;

  case ps_down1:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_quiet,T_2,0,0);
    if (padd & PER_DGT_KD) player_set_state_and_pos(p,ps_down2,T_2,0,0);
    }break;
  case ps_down2:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_down3,T_2,0,0);
    }break;
  case ps_down3:
    if  (padp & K_RUN){ 
      if (p->level!=0) {
	player_set_state_and_pos(p,ps_hang1,T_2,0,-LEVEL_HEIGHT+toFIXED(32.0));
	player_change_level(p,-1);
      } else (p->st_timeout)++;
    }
    else if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_up1,T_2,0,0);
    if (padd & PER_DGT_KR) {
      player_return(p);
      player_set_state_and_pos(p,ps_down2,T_2,0,0);
    }
    else if (padd & K_FOOT)  player_set_state_and_pos(p,ps_downhit1,T_2,0,0);
    else if (padd & PER_DGT_KD) player_set_state_and_pos(p,ps_down3,T_2,0,0);
    }break;
    

  case ps_up1:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_up2,T_2,0,0); 
    }break;
  case ps_up2:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_quiet,T_2,0,0);
    }break;
  

  case ps_hikick1:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_hikick2,T_2,0,0);
    }break;
  case ps_hikick2:
    if ((p->st_timeout) == 0){   
      player_set_state_and_pos(p,ps_hikick3,T_2,0,0);
      sound_tell(vs_missed,20);
    }break;
  case ps_hikick3:
    if ((p->st_timeout) == 0){ 
      if (padd & PER_DGT_KR) {player_return(p); player_set_state_and_pos(p,ps_hikick3,T_2,0,0);}
      else player_set_state_and_pos(p,ps_hikick4,T_2,toFIXED(-2.0),0);
    }break;
  case ps_hikick4:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_hikick5,T_2,0,0);
    }break;
  case ps_hikick5:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_quiet,T_2,0,0);
    }break;


  case ps_walk1:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_walk2,T_2,toFIXED(-4.0),0);
    }break;
  case ps_walk2:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_walk3,T_2,toFIXED(-3.0),0);
    if (padd & PER_DGT_KR) player_set_state_and_pos(p,ps_quiet,T_2,toFIXED(2.0),0);
    else if (padd & PER_DGT_KL) player_set_state_and_pos(p,ps_walk3,T_2,toFIXED(-3.0),0);
    sound_tell(minmaxrndint(vs_step1,vs_step6),11);
    }
    break;
  case ps_walk3:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_quiet,T_2,toFIXED(-3.0),0);
    }break;

  case ps_downhit1:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_downhit2,T_2,0,0);
    }break;
  case ps_downhit2:
    if ((p->st_timeout) == 0){ 
      player_set_state_and_pos(p,ps_downhit3,T_2,0,0);
      sound_tell(vs_missed,20);
    }break;
  case ps_downhit3:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_downhit4,T_2,0,0);
    }break;
  case ps_downhit4:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_down3,T_2,0,0);
    }break;



  case ps_jump1:
    if ((p->st_timeout) == 0){ 
      if (padd & PER_DGT_KR) player_return(p);
      player_set_state_and_pos(p,ps_jump2,T_2,0,toFIXED(-5.0));
    }break;
  case ps_jump2:
    if ((p->st_timeout) == 0){ 
      if (padd & PER_DGT_KR) player_return(p);
      player_set_state_and_pos(p,ps_jump3,T_2,0,toFIXED(-12.0));
    }break;
  case ps_jump3:
    if ((p->st_timeout) == 0){ 
      if (padd & K_RUN) 
	if (p->level!=N_LEVEL-1) {
	  player_set_state_and_pos(p,ps_hang1,T_2,0,-LEVEL_HEIGHT+toFIXED(32.0)+toFIXED(17.0));
	} else {
	  p->st_timeout++;
	}
      else if (padd & K_FOOT) player_set_state_and_pos(p,ps_hf1,T_3,0,0);    
      else player_set_state_and_pos(p,ps_jump4,T_2,0,toFIXED(2.0));
    }break;
    
  case ps_jump4:
    if ((p->st_timeout) == 0){ 
      if (padd & PER_DGT_KR) player_return(p);
      player_set_state_and_pos(p,ps_jump5,T_2,0,toFIXED(15.0));
    }break;
  case ps_jump5:
    if ((p->st_timeout) == 0){ player_set_state_and_pos(p,ps_quiet,T_2,3,toFIXED(0.0));
    }break;

  case  ps_levelup1:
    if ((p->st_timeout) == 0){    
      player_set_state_and_pos(p,ps_levelup2,T_2,toFIXED(-4.0),toFIXED(-3.0));
    } break;
  case  ps_levelup2:
    if ((p->st_timeout) == 0){
      player_set_state_and_pos(p,ps_levelup3,T_2,toFIXED(-8.0),toFIXED(-17.0));
    } break;
  case  ps_levelup3:
    if ((p->st_timeout) == 0){
      player_set_state_and_pos(p,ps_levelup4,T_2,toFIXED(4.0),toFIXED(-15.0));
    } break;
  case  ps_levelup4:
    if ((p->st_timeout) == 0){
      player_change_level(p,1);
      player_set_state_and_pos(p,ps_levelup5,T_2,toFIXED(2.0),toFIXED(2.0)+LEVEL_HEIGHT);
    } break;
  case  ps_levelup5:
    if ((p->st_timeout) == 0){
      player_set_state_and_pos(p,ps_down3,T_2,0,toFIXED(33.0)-toFIXED(32.0));
    } break;
    

    

  case ps_hang1 :
    if (padp & PER_DGT_KU) player_set_state_and_pos(p,ps_levelup1,T_2,0,0);
    else if (padp & PER_DGT_KD) player_set_state_and_pos(p,ps_down2,T_2,0,LEVEL_HEIGHT-toFIXED(32.0));
    else if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR) {player_return(p); player_set_state_and_pos(p,ps_hang2,T_3,0,0);}
      else player_set_state_and_pos(p,ps_hang2,T_3,0,0);
    }
    break;
  case ps_hang2 :
    if (padp & PER_DGT_KU) player_set_state_and_pos(p,ps_levelup1,T_2,0,0);
    else if (padp & PER_DGT_KD) player_set_state_and_pos(p,ps_down2,T_2,0,LEVEL_HEIGHT-toFIXED(32.0));
    else if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR) {player_return(p); player_set_state_and_pos(p,ps_hang3,T_3,0,0);}
      else player_set_state_and_pos(p,ps_hang3,T_3,0,0);
    }
    break;
  case ps_hang3 :
    if (padp & PER_DGT_KU) player_set_state_and_pos(p,ps_levelup1,T_2,0,0);
    else if (padp & PER_DGT_KD) player_set_state_and_pos(p,ps_down2,T_2,0,LEVEL_HEIGHT-toFIXED(32.0));
    else if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KL)  {
	player_set_state_and_pos(p,ps_hangs1,T_3,0,0);
      }
      else player_set_state_and_pos(p,ps_hang4,T_3,0,0);
    }
    break;
  case ps_hang4 :
    if (padp & PER_DGT_KU) player_set_state_and_pos(p,ps_levelup1,T_2,0,0);
    else if (padp & PER_DGT_KD) player_set_state_and_pos(p,ps_down2,T_2,0,LEVEL_HEIGHT-toFIXED(32.0));
    else if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KL)  {
	player_set_state_and_pos(p,ps_hangs1,T_3,0,0);
	
      }
      else player_set_state_and_pos(p,ps_hang1,T_3,0,0);
    }
    break;
    


  case ps_hihand1 : 
    if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR) player_return(p);
      player_set_state_and_pos(p,ps_hihand2,T_2,toFIXED( 1.0),0);    

    }break;
  case ps_hihand2 : 
    if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR) player_return(p);
      player_set_state_and_pos(p,ps_hihand3,T_2,toFIXED(-1.0),0); 
    }break;
  case ps_hihand3 : 
    if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR) player_return(p);
      player_set_state_and_pos(p,ps_hihand4,T_2,toFIXED(-4.0),0);    
      sound_tell(vs_missed,20);
    }break;
  case ps_hihand4 : 
    if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR) player_return(p);
      player_set_state_and_pos(p,ps_hihand5,T_2,toFIXED(-1.0),0);      
    }break;
  case ps_hihand5 : 
    if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR) player_return(p);
      if (padd & K_HAND) player_set_state_and_pos(p,ps_hihand1,T_2,toFIXED(-5.0),0);
      else player_set_state_and_pos(p,ps_quiet,T_2,toFIXED(0.0),0);      
    }break;
 
  case ps_hf1 :
    if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR)  player_return(p);
      player_set_state_and_pos(p,ps_hf2,T_2,0,0);
    }
    break;
  case ps_hf2 :
    if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR)  player_return(p);
      player_set_state_and_pos(p,ps_hf3,T_3,0,0);
      sound_tell(vs_missed,20);
    }
    break;
  case ps_hf3 :
    if ((p->st_timeout) == 0)
      if (padd & PER_DGT_KR)  player_return(p);
      player_set_state_and_pos(p,ps_hf4,T_2,0,toFIXED(5.0));
    break;
  case ps_hf4 :
    if ((p->st_timeout) == 0)
      if (padd & PER_DGT_KR)  player_return(p);
      player_set_state_and_pos(p,ps_quiet,T_2,0,toFIXED(12.0));
    break;

  case ps_nsh1 :
    if ((p->st_timeout) == 0){
      player_set_state_and_pos(p,ps_nsh2,T_2,0,toFIXED(0.0));
    }
    break;
  case ps_nsh2 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_nsh3,T_2,0,toFIXED(0.0));
    break;
  case ps_nsh3 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_nsh4,T_2,0,toFIXED(-2.0));
    break;
  case ps_nsh4 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_nsh5,T_300,0,toFIXED(2.0));
    break;
    player_fall(p);
  case ps_nsh5 :
    if ((p->st_timeout) == 0){
      if (ga & GA_NEVER_DIE) {
	player_set_state_and_pos(p,ps_quiet,T_2,0,toFIXED(0.0));
	p->energy = MAX_ENERGY/(p->ndied+1);
      } else if (ga & GA_TEAM){
	player_set_state_and_pos(p,ps_quiet,T_2,0,toFIXED(0.0));
	p->energy = MAX_ENERGY;
      }
    }
    player_fall(p);
    break;

  case ps_sahl1 :
    if ((p->st_timeout) == 0){
      player_set_state_and_pos(p,ps_sahl2,T_3,0,0);
    } break;
  case ps_sahl2 :
    if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KR) player_return(p);
      //      if (padd & K_PROTECT) player_set_state_and_pos(p,ps_sahl2,T_2,0,0);
      else player_set_state_and_pos(p,ps_quiet,T_2,0,0);
    } break;
 
    

  case ps_hihhr1 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_hihhr2,T_2,toFIXED(-10.0),toFIXED(-22.0));
    break;
  case ps_hihhr2 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_hihhr3,T_2,toFIXED(-8.0),toFIXED(3.0));
    break;
  case ps_hihhr3 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_hihhr4,T_2,toFIXED(-7.0),toFIXED(3.0));
    break;
  case ps_hihhr4 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_hihhr5,T_2,toFIXED(-6.0),toFIXED(6.0));
    break;
  case ps_hihhr5 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_run2,T_1,toFIXED(-4.0),toFIXED(10.0));
    break;


  case ps_gardarr1 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_gardarr2,T_1,toFIXED(0.0),toFIXED(0.0));
    break;
  case ps_gardarr2 :
    if ((p->st_timeout) == 0){
      if (padd & K_FOOT) {
	player_set_state_and_pos(p,ps_hikick1,T_2,0,0);
      }else{
	player_set_state_and_pos(p,ps_gardarr3,T_1,toFIXED(4.0),toFIXED(0.0));
      }
    }
    break;
  case ps_gardarr3 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_gardarr4,T_1,toFIXED(0.0),toFIXED(0.0));
    break;
  case ps_gardarr4 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_gardarr5,T_1,toFIXED(1.0),toFIXED(0.0));
    break;
  case ps_gardarr5 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_gardarr6,T_1,toFIXED(2.0),toFIXED(0.0));
    break;
  case ps_gardarr6 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_quiet,T_1,toFIXED(1.0),toFIXED(0.0));
    break;


  case ps_pfixe1 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_pfixe2,T_3,toFIXED(-1.0),toFIXED(0.0));
    break;
  case ps_pfixe2 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_pfixe3,T_3,toFIXED(0.0),toFIXED(0.0));
    break;
  case ps_pfixe3 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_pfixe4,T_3,toFIXED(0.0),toFIXED(0.0));
    break;
  case ps_pfixe4 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_quiet,T_1,toFIXED(-1.0),toFIXED(0.0));
    break;

  case ps_hangs1 :
    if ((p->st_timeout) == 0){
      if (padd & PER_DGT_KD) {
	player_set_state_and_pos(p,ps_down2,T_2,0,LEVEL_HEIGHT-toFIXED(32.0));
      } else if (padp & PER_DGT_KU) {
	player_set_state_and_pos(p,ps_levelup1,T_2,0,0);
      } else {
	player_set_state_and_pos(p,ps_hangs2,T_3,toFIXED(0.0),toFIXED(0.0));
      }
    }
    break;
  case ps_hangs2 :
    if ((p->st_timeout) == 0){
      if (!((padd & PER_DGT_KL)||(padd & PER_DGT_KR))){
	player_set_state_and_pos(p,ps_hang1,T_2,0,0);
      } else {
	player_set_state_and_pos(p,ps_hangs3,T_3,toFIXED(0.0),toFIXED(0.0));
      }
    }
    break;
  case ps_hangs3 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_hangs4,T_3,toFIXED(0.0),toFIXED(0.0));
    break;
  case ps_hangs4 :
    if ((p->st_timeout) == 0){
      if (padd & K_FOOT){
	player_set_state_and_pos(p,ps_hihhr3,T_3,toFIXED(-18.0),toFIXED(18.0));
      } else {
	player_set_state_and_pos(p,ps_hangs5,T_3,toFIXED(0.0),toFIXED(0.0));
      }
    }
    break;
  case ps_hangs5 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_hangs6,T_3,toFIXED(0.0),toFIXED(0.0));
    break;
  case ps_hangs6 :
    if ((p->st_timeout) == 0)
      player_set_state_and_pos(p,ps_hangs1,T_3,toFIXED(0.0),toFIXED(0.0));
    break;


  default:
    break;
  }

  p->st_timeout--;
}



/* GAME */


void game_reinit(GAME* g){
  int i;
  for (i=0;i<MAX_PLAYER;i++){
    player_reinit(&(g->p[i]));
  }
  if (g->flags & GA_TIME_LIMIT){
    g->t_limit = DEFAULT_TIME_LIMIT;
  }
  for (i=0;i<MAX_ITEM;i++)
    item_reinit(&(g->it[i]));
  
  camion_reinit(&(g->c));
}

void game_init(GAME* g){
  int i;
  
  g->flags = GA_TIME_LIMIT;
  for (i=0;i<MAX_PLAYER;i++){
    player_init(&(g->p[i]));
    g->p[i].gr_col = GRaddr+i;
    mark_init(&(g->p[i].m),WHITE_MARK);
  }
  expl_init_all();
  for (i=0;i<MAX_ITEM;i++)
    item_init(&(g->it[i]));
  camion_init(&(g->c));
  game_reinit(g);
}


int game_can_begin(GAME* g){
  int i;

  int n_player=0;

  for (i=0;i<MAX_PLAYER;i++){
    n_player += (g->p[i].active)?1:0;
  }

  return (n_player);
}


int game_one_team_remains(GAME* g){
  int first_team = -1;
  int cur = 0;
  int count = 0;
  do {
    PLAYER* p = &(g->p[cur]);
    if (p->active){
      if (!player_is_ko(p)){
	if (p->team != first_team){
	  count++;
	  if (first_team == -1)
	    first_team = p->team;
	}
      }
    }
    cur++;
  } while ((count<=1)&&(cur<MAX_PLAYER));

  return (count == 1);
}

int game_one_player_remains(GAME* g){
  int count=0;
  int cur=0;
  do {
    PLAYER* p = &(g->p[cur]);
    if (p->active)
      count += player_is_ko(p)?0:1;
    cur++;
  } while ((count<=1)&&(cur<MAX_PLAYER));
  return (count == 1);
}

int game_can_end(GAME* g){

  int end = FALSE;


  if (g->flags & GA_TIME_LIMIT){
    end = !(g->t_limit--);
    if (g->t_limit == (50*3)) sound_tell(vs_g_end,0);
    else if (g->t_limit == (50*30)) sound_tell(vs_g_30s,0);
  }

  if (!end){
    if (g->flags & GA_TEAM){
      end = game_one_team_remains(g);
    } else {
      end = game_one_player_remains(g);
    }
  }

  return (end);
}

void game_item_set(GAME* g, FIXED posx, int level, ITEM_CLASS type){
  int curit = 0;
  int found = FALSE;
  ITEM* it;

  do {
    it = &(g->it[curit]);
    found = ((it->active == FALSE));
    curit++;
  } while ((curit<MAX_ITEM)&&(!found));

  if (found)
    item_set(it,posx,level,type);
}


void game_check_items(GAME* g){

  int i;
  for (i=0; (i<MAX_PLAYER); i++){
    PLAYER* p = &(g->p[i]);
    if (p->active){
      int j;
      for (j=0;j<MAX_ITEM;j++){
	ITEM* it = &(g->it[j]);
	if ((it->nplayer == -1)&&(it->active)){
	  if (player_can_take(p,it->posx,it->level)){
	    g->p[i].energy = MAX_ENERGY;
	    it->nplayer = i;
	    // quick hack
	    it->active = FALSE;
	    
	    sound_tell(vs_ressort,1);
	  }
	}
      }
    }
  }
}



void game_check_camion(GAME* g){

  int i;
  FIXED bidon;
  CAMION* c = &(g->c);
  
  for (i = 0; (i<MAX_PLAYER); i++){
    PLAYER* p = &(g->p[i]);
    if (p->active){
      switch (p->state) {

      case ps_hang1: case  ps_hang2: case ps_hang3 : case ps_hang4 :
      case ps_hangs1: case  ps_hangs2: case ps_hangs3 : case ps_hangs4 :case ps_hangs5: case  ps_hangs6:
	if (camion_hits(c,p->posx,p->level+1,&bidon)){
	  player_set_state_and_pos(p,ps_down2,T_2,0,LEVEL_HEIGHT-toFIXED(32.0));
	  sound_tell(vs_hit3,3);
	}	
	break;
      case ps_nsh4 : case ps_nsh5 :
	break;
      case ps_levelup1 : case ps_levelup2 : case ps_levelup3 : case ps_levelup4 :
	break;
      default :
	if (camion_hits(c,p->posx,p->level,&bidon)){
	  player_set_state_and_pos(p,ps_nsh4,T_1,0,0);
	  sound_tell(vs_crunch,1);
	}
      }
    }
  }

}

Uint32 game_set_closer_player(GAME *g,PLAYER* p,PLAYER** cl){
  int i;
  FIXED dist=toFIXED(100.0);
  (*cl) = (PLAYER*)0;
  FIXED detection_range = toFIXED(100.0);

  for (i=0;i<MAX_PLAYER;i++){
    PLAYER* cur = &(g->p[i]);
    if ((cur->active)&&(cur!=p)&&(cur->state != ps_nsh5)){
      if (p->level == cur->level){
	FIXED curdist = cur->posx - p->posx;
	if ((-detection_range<curdist)&&(curdist<detection_range)){
	  if (slMulFX(dist,dist)>slMulFX(curdist,curdist)){
	    dist = curdist ;
	    (*cl) = cur;
	  }
	}
      }
    }
  }
  return (Uint32)dist;
}


/*
void ground_draw(int scr){
  int i;
  int colb = 50;
  

  scr++;
  for (i=0;i<N_LEVEL;i++){
    int j;
    int y= (ground_height[i]>>16)+16+1;
    for (j=0;j<5;j++)
      if (scr<2){
	slBMLine(320*scr,120+y+j,320*scr+320,120+y+j,colb);
      } else {
	slBMLine(320*(scr-2),240+120+y+j,320*(scr-2)+320,240+120+y+j,colb);
      }
  }
}
*/
