#include <sddrvs.dat>
#include "libpad.h"
#include "sgl.h"
#include "sound.h"
#include "sounds/g_30s.c"
#include "sounds/g_end.c"
#include "sounds/hit1.c"
#include "sounds/hit2.c"
#include "sounds/hit3.c"
#include "sounds/die.c"
#include "sounds/missed.c"
#include "sounds/step1.c"
#include "sounds/step2.c"
#include "sounds/step3.c"
#include "sounds/step4.c"
#include "sounds/step5.c"
#include "sounds/step6.c"
#include "sounds/ressort.c"
#include "sounds/crunch.c"
#include "sounds/sirene.c"
#include "sounds/camion.c"
#include "sounds/sine.c"


static Uint32 soffset = 0x0;//0x2C;
SOUND_PCM vs_sound[vs_last];
#define SOUND_N_CHANNEL 4
SOUND_CHANNEL_STATE channel_state[SOUND_N_CHANNEL];

void sound_init()
{
  char sound_map[] = {0xff , 0xff} ;
  slInitSound(sddrvstsk , sizeof(sddrvstsk) , (Uint8 *)sound_map , sizeof(sound_map)) ;
  slSndVolume(127);
  sound_init_sounds();
}

void sione(SOUND_TYPE st, char* d, Uint32 s, PCM* info){
  vs_sound[st].data = d+soffset;
  vs_sound[st].size = s-soffset;
  vs_sound[st].info  = info;
}

void sound_init_sounds(){  
  sione(vs_g_30s,g_30s,g_30s_size,&g_30s_dat);
  sione(vs_g_end,g_end,g_end_size,&g_end_dat);
  sione(vs_hit1,hit1,hit1_size,&hit1_dat);  
  sione(vs_hit2,hit2,hit2_size,&hit2_dat);  
  sione(vs_hit3,hit3,hit3_size,&hit3_dat);  
  sione(vs_die,die,die_size,&die_dat);  
  sione(vs_missed,missed,missed_size,&missed_dat);  
  sione(vs_step1,step1,step1_size,&step1_dat);  
  sione(vs_step2,step2,step2_size,&step2_dat);  
  sione(vs_step3,step3,step3_size,&step3_dat);  
  sione(vs_step4,step4,step4_size,&step4_dat);  
  sione(vs_step5,step5,step5_size,&step5_dat);  
  sione(vs_step6,step6,step6_size,&step6_dat);  
  sione(vs_ressort,ressort,ressort_size,&ressort_dat);  
  sione(vs_crunch,crunch,crunch_size,&crunch_dat);  
  sione(vs_sirene,sirene,sirene_size,&sirene_dat);  
  sione(vs_camion,camion,camion_size,&camion_dat);  
  sione(vs_sine,sine,sine_size,&sine_dat);
}

void sound_channel_print(int x, int y){
  int i;
  for (i=0;i<SOUND_N_CHANNEL;i++){
    if (channel_state[i].type!=-1){
      slPrintHex(channel_state[i].type,slLocate(x,y+i));
      slPrintHex(channel_state[i].prio,slLocate(x+8,y+i));
      slPrintHex(channel_state[i].age ,slLocate(x+16,y+i));
    } else {
      slPrint("|              ",slLocate(x,y+i));
      slPrint("               ",slLocate(x+8,y+i));
      slPrint("               ",slLocate(x+16,y+i));
    }
  }
}

void sound_step(){
  int i;
  for (i=0;i<SOUND_N_CHANNEL;i++){
    channel_state[i].age +=1;
  }
  sound_channel_state_get();
}

void sound_channel_state_get(){
  static PCM sndo_dat = {(_Mono | _PCM8Bit) , 0 , 127 , 0 , 0x69ce , 0,0,0,0};

  int i;
  for (i=0;i<SOUND_N_CHANNEL;i++){
    sndo_dat.channel = i;
    if (!slPCMStat(&sndo_dat))
      channel_state[i].type = -1;
  }
}

int sound_get_least_prio(int* chan){
  int i;
  int c,p,age;
  c = -1;
  p = -1;
  age = -1;
  sound_channel_state_get();

  for (i=0;i<SOUND_N_CHANNEL;i++){
    if (channel_state[i].type == -1){
      return (-1);
    }
    if (p<=channel_state[i].prio){
      if (channel_state[i].age>age){
	p = channel_state[i].prio;
	c = i;
	age = channel_state[i].age;
      }
    }
  }
  *chan = c;
  return p;
}

void sound_tell(SOUND_TYPE st, int prio){
  PCM * dpcm = vs_sound[st].info;
  sound_tell_more(st,prio,dpcm->level,dpcm->pan,(dpcm->pitch));
}


void sound_test(){
  int cur1=0;
  int cur2=0;
  
  int st=0;
  int prio=0;
  Uint8 level=127;
  Sint8 pan = 0;
  Uint16 pitch = 0x69ce;

  for (;;){

    sound_channel_print(10,20);
    sound_channel_state_get();
    sound_step();

    cur2=0;

    if (pad_someone_presses(PER_DGT_KU, TRUE)) cur1--;
    if (pad_someone_presses(PER_DGT_KD, TRUE)) cur1++;
    if (pad_someone_presses(PER_DGT_KL, TRUE)) cur2=-1;
    if (pad_someone_presses(PER_DGT_KR, TRUE)) cur2=1;

    int y=5;
    if (cur1<0) cur1=0;
    if (cur1>4) cur1=4;

    if (cur1==0){
      prio += cur2;
    } else if (cur1==1){
      level+= cur2;
    } else if (cur1==2){
      pan +=cur2;
    } else if (cur1==3){
      pitch +=cur2;
    } else if (cur1==4){
      st +=cur2;
    }

    slPrint(" ",slLocate(1,y));
    slPrint("prio",slLocate(3,y));
    slPrintHex(prio,slLocate(20,y++));

    slPrint(" ",slLocate(1,y));
    slPrint("level",slLocate(3,y));
    slPrintHex(level,slLocate(20,y++));

    slPrint(" ",slLocate(1,y));
    slPrint("pan",slLocate(3,y));
    slPrintHex(pan,slLocate(20,y++));

    slPrint(" ",slLocate(1,y));
    slPrint("pitch",slLocate(3,y));
    slPrintHex(pitch,slLocate(20,y++));

    slPrint(" ",slLocate(1,y));
    slPrint("type",slLocate(3,y));
    sound_pcm_display_name(st,20,y++);    

    slPrint(">",slLocate(1,5+cur1));
 
    if (pad_someone_presses(PER_DGT_TA, TRUE))
      sound_tell_more(st,prio,level,pan,pitch);



    
    slSynch();
  }

}

void sound_tell_more(SOUND_TYPE st, int prio, Uint8 level, Sint8 pan, Uint16 pitch){
  
  PCM * dpcm = vs_sound[st].info;
  void* laddr = vs_sound[st].data;
  Uint32 lsize = vs_sound[st].size;
  PCM lpcm;
  
  if (st==vs_none) return;

  lpcm.mode    = dpcm->mode ;
  lpcm.channel = dpcm->channel;
  lpcm.pan     = pan;

  lpcm.pitch   = pitch;
  lpcm.level   = level;

  int channel  =  slPCMOn(&lpcm,laddr,lsize);

  if (channel>=0){
    channel_state[channel].type = st;
    channel_state[channel].prio = prio;
    channel_state[channel].age = 0;
  } else {
    int cut;
    if (sound_get_least_prio(&cut)>=prio){
      lpcm.channel = cut;
      slPCMOff(&lpcm);
      sound_tell_more(st, prio,level,pan,pitch);
    }
  }
}

void sound_pcm_all_off(){
  PCM lpcm;
  int i;
  for (i=0;i<SOUND_N_CHANNEL;i++){
    lpcm.channel = i;
    slPCMOff(&lpcm);
  }
  sound_channel_state_get();
}

void sound_pcm_gfx_display(SOUND_TYPE st,int x1, int y1, int x2, int y2){
  
  int i;

  slBMBoxFill(x1,y1,x2,y2,0);

  int dx = x2-x1;
  int dy = y2-y1;
  int ycenter = y1+dy/2;

  for (i=0;i<dx;i++){
    Uint8* add = (Uint8*) ((vs_sound[st].data + (vs_sound[st].size*i)/dx));
    Sint8 s = *add;
    int col = (int)add%256;
    slBMLine(x1+i,ycenter,x1+i,ycenter+s*dy/128,col);
  }
  
}

void sound_pcm_display_name(SOUND_TYPE st,int x, int y){
  switch(st){
  case vs_g_30s  : slPrint("vs_g_30s   ",slLocate(x,y)); break;  
  case vs_g_end  : slPrint("vs_g_end   ",slLocate(x,y));  break;  
  case vs_camion : slPrint("vs_camion  ",slLocate(x,y));  break;  
  case vs_hit1   : slPrint("vs_hit1    ",slLocate(x,y));  break;  
  case vs_hit2   : slPrint("vs_hit2    ",slLocate(x,y));  break;  
  case vs_hit3   : slPrint("vs_hit3    ",slLocate(x,y));  break;  
  case vs_die    : slPrint("vs_die     ",slLocate(x,y));  break;  
  case vs_missed : slPrint("vs_missed  ",slLocate(x,y));  break;  
  case vs_step1  : slPrint("vs_step1   ",slLocate(x,y));  break;  
  case vs_step2  : slPrint("vs_step2   ",slLocate(x,y));  break;  
  case vs_step3  : slPrint("vs_step3   ",slLocate(x,y));  break;  
  case vs_step4  : slPrint("vs_step4   ",slLocate(x,y));  break;  
  case vs_step5  : slPrint("vs_step5   ",slLocate(x,y));  break;  
  case vs_step6  : slPrint("vs_step6   ",slLocate(x,y));  break;  
  case vs_ressort: slPrint("vs_ressort ",slLocate(x,y));  break;  
  case vs_crunch : slPrint("vs_crunch  ",slLocate(x,y));  break;  
  case vs_sirene : slPrint("vs_sirene  ",slLocate(x,y));  break;  
  case vs_sine   : slPrint("vs_sine    ",slLocate(x,y));  break;
  case vs_last   : slPrint("vs_last    ",slLocate(x,y));  break;  
  default   : slPrint("???        ",slLocate(x,y));  break;  
    
  }
  slPrintHex(vs_sound[st].size,slLocate(x,y+1));
}

